import { Injectable } from '@nestjs/common';
import { mkdirSync } from 'fs';
import * as fs from 'fs';
import puppeteer from 'puppeteer';
import { URL } from 'url';
import axios from 'axios';
import { Site } from '../interfaces/site.interface';

@Injectable()
export class CrawlerService {

  constructor() {
    if (!fs.existsSync('sites_assets')) {
      mkdirSync('sites_assets');
    }
  }

  async crawl(url: string): Promise<Site> {

    console.log("start crawl website", url);
    const browser = await puppeteer.launch({ executablePath: '/usr/bin/chromium-browser', headless: true, args: ['--no-sandbox'] });
    const page = await browser.newPage();
    const domain = this.extractDomain(url);
    await page.goto(url);
    const directory = `sites_assets/${domain}/`;
    if (!fs.existsSync(directory)) {
      mkdirSync(directory);
    }

    // STYLESHEETS //
    console.log("start stylesheets")
    const stylesheetsUrls = await page.$$eval('link[rel="stylesheet"]', links => links.map(link => link.href));
    let cssDir = `${directory}/css/`
    const cssSheetsLocation = await this.downloadFiles(stylesheetsUrls, cssDir);
    console.log(`cssSheetsLocation: `, cssSheetsLocation);
    // STYLESHEETS //

    // SCRIPTS //
    const scriptsUrls = await page.$$eval('script', scripts => scripts.map(script => script.src));
    let scriptsDir = `${directory}/scripts/`
    const scriptsSheetsLocation = await this.downloadFiles(scriptsUrls, scriptsDir);
    console.log(`scriptsSheetsLocation: `, scriptsSheetsLocation)
    // SCRIPTS //

    // SCREENSHOT //
    const screenshotBuffer: Buffer = await page.screenshot({ fullPage: true });
    await new Promise((resolve, reject) => {
      fs.writeFile(`${directory}screenshot.png`, screenshotBuffer, (err) => {
        if (err) {
          reject(err);
        } else {
          resolve(true);
        }
      });
    });
    // SCREENSHOT //

    // URLS //
    const urls = await page.$$eval('a', links => links.map(link => link.href));
    const urlsList = urls.filter((url) => url.startsWith('http'));
    console.log(urlsList);
    // URLS //

    await browser.close();
    return {
      domain,
      cssSheetsLocation,
      scriptsSheetsLocation,
      urlsList
    }
  }

  async downloadFiles(urls: string[], path: string) {
    const finalUrls: string[] = [];
    if (!fs.existsSync(path)) {
      mkdirSync(path);
    }

    await Promise.all(
      urls.map(async (url) => {
        if (!url.startsWith('http')) return;
        const response = await axios.get(url);
        const content = response.data;
        // trim / from end of url string
        let fileLocation = url.replace(/\/$/, "");
        // get last part of url
        fileLocation = fileLocation.substring(fileLocation.lastIndexOf('/') + 1);
        // save file
        if (fileLocation.length > 10) {
          fileLocation = fileLocation.substring(0, 10);
        }
        finalUrls.push(`${path}${fileLocation}`);
        console.log(`Saving file ${path}${fileLocation}`);
        fs.writeFileSync(`${path}${fileLocation}`, content);
      })
    );

    return finalUrls;
  }

  extractDomain(urlString: string) {
    const url = new URL(urlString);
    return url.hostname;
  }
}
